﻿using DbCommon.Enties.DbModels;
using ProjectManagementSystem.Common.Config;
using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models;
using ProjectManagementSystem.Common.Models.Crms;
using ProjectManagementSystem.Language;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Service
{
    public class PmsTaskServiceSWms : PmsTaskService
    {
        public override WebMgrResponse GetLocationProperty(LocationPropertyQueryDto data)
        {
            WebMgrResponse webMgrResponse = new WebMgrResponse();

            //查询信息
            var locationStocks = Crms.SWmsApi.GetLocationStock(data.WareHouseCode, data.Area, data.LocationCode);
            if (locationStocks == null)
            {
                webMgrResponse.message = Langs.Failed;
                return webMgrResponse;
            }

            var dataList = locationStocks
                .OrderBy(d => d.Location.warehouseId)
                .ThenBy(d => d.Location.locationId.Length)
                .ThenBy(d => d.Location.locationId)
                .ToList();

            var excelConfig = ExcelConfig.Instance;
            var dtoList = new List<LocationPropertyDto>(dataList.Count);
            foreach (var item in dataList)
            {
                LocationPropertyDto dto = new LocationPropertyDto();
                dto.WareHouseCode = item.Location.warehouseId;
                dto.LocationCode = item.Location.locationId;
                dto.Mark = excelConfig.GetRouteConfig(dto.LocationCode)?.StationID ?? 0;
                dto.Area = item.Location.areaId;
                //Enum.TryParse(item.Location.locationStatus, out LocationStatus locationStatus);
                //dto.Status = locationStatus;

                dto.Status = item.Location.locationStatus == "0" ? LocationStatus.None : item.Stock == null ? LocationStatus.Empty : LocationStatus.Filled;
                dto.MaterialId = item.Stock?.goodsCode;
                dto.MaterialNumber = item.Stock?.goodsNumber.ToString();
                dto.CurrentTaskID = item.Location.lockMark;

                dto.TargetWareHouseCode = dto.LocationCode.GetLocationMember("目标仓库");
                dto.SourceWareHouseCode = dto.LocationCode.GetLocationMember("源仓库");
                dto.MaterialIdCollection = dto.LocationCode.GetLocationMember("可选物料");
                dtoList.Add(dto);
            }

            webMgrResponse.code = 20000;
            webMgrResponse.data = dtoList;
            webMgrResponse.message = Langs.Successful;
            return webMgrResponse;
        }
        public override WebMgrResponse UpdateLocationProperty(LocationPropertyUpdateDto data)
        {
            lock (locker)
            {
                WebMgrResponse webMgrResponse = new WebMgrResponse();

                //查询信息
                var locationData = Crms.SWmsApi.GetLocationByCondition(new SWmsLocationByCondition() { locationId = data.LocationCode }, out var content)?.FirstOrDefault();
                if (locationData == null)
                {
                    webMgrResponse.message = $"{data.LocationCode} {Langs.LocationGetInfoFailed}";
                    return webMgrResponse;
                }

                //有任务的位置不能修改
                if (!string.IsNullOrEmpty(locationData.lockMark))
                {
                    if (LocationHasStepTask(data.LocationCode, out string taskId))
                    {
                        webMgrResponse.message = $"{data.LocationCode} {Langs.LocationOccupiedByTask} {taskId}";
                        return webMgrResponse;
                    }
                }

                bool result = false;

                //处理未知状态
                var statusCode = data.Status == LocationStatus.None ? data.Status.ToValue<int>().ToString() : string.Empty;
                if (locationData.locationStatus != statusCode)
                {
                    locationData.locationStatus = statusCode;
                    result = Crms.SWmsApi.UpdateLocation(locationData, out content);
                    if (!result)
                    {
                        webMgrResponse.message = $"{data.LocationCode} {Langs.LocationStatusUpdateFailed} {(result ? null : content)}";
                        return webMgrResponse;
                    }
                }

                //解锁库位
                result = Crms.SWmsApi.UpdateLocationLockMark(data.LocationCode, locationData.lockMark, null, out content);
                if (!result)
                {
                    webMgrResponse.message = $"{data.LocationCode} {Langs.LocationUnLockFailed} {(result ? null : content)}";
                    return webMgrResponse;
                }

                //根据状态设置货物
                if (data.Status == LocationStatus.Filled && string.IsNullOrWhiteSpace(data.MaterialId))
                {
                    data.MaterialId = "默认货物";
                }
                if (data.Status == LocationStatus.Empty && !string.IsNullOrWhiteSpace(data.MaterialId))
                {
                    data.MaterialId = null;
                }

                //货物数量转换
                var materialNumber = 1;
                if (!string.IsNullOrWhiteSpace(data.MaterialNumber) && !int.TryParse(data.MaterialNumber, out materialNumber))
                {
                    webMgrResponse.message = $"{Langs.MaterialNumber} {Langs.ConversionFailed} {data.MaterialNumber}";
                    return webMgrResponse;
                }

                SWmsStock stock = new SWmsStock();
                stock.recordType = SWmsERecordType.ManualIn;
                stock.supporterType = SWmsESupporterType.Location;
                stock.supporterId = data.LocationCode;
                stock.goodsCode = data.MaterialId;
                stock.goodsNumber = materialNumber;
                stock.operationSource = "pms";

                result = Crms.SWmsApi.StockSave(stock, out var message);
                CLog.Instance.SystemLog.WirteInfoCaller(message);

                webMgrResponse.code = result ? 20000 : 9999;
                webMgrResponse.message = message;
                return webMgrResponse;
            }
        }
        public override WebMgrResponse BindAgvMaterial(AgvMaterialDto data)
        {
            SWmsStock stock = new SWmsStock();
            stock.recordType = SWmsERecordType.ManualIn;
            stock.supporterType = SWmsESupporterType.Carrier;
            stock.supporterId = data.AgvId.ToString();
            stock.goodsCode = data.MaterialId;
            stock.goodsNumber = 1;
            stock.operationSource = "pms";
            stock.attribute1 = data.MaterialBarcode;

            var result = Crms.SWmsApi.StockSave(stock, out var message);
            CLog.Instance.SystemLog.WirteDebugCaller(message);

            WebMgrResponse res = new WebMgrResponse();
            res.code = result ? 20000 : 9999;
            res.message = message;
            return res;
        }
        public override WebMgrResponse GetAgvMaterial(AgvMaterialDto data)
        {
            WebMgrResponse res = new WebMgrResponse();

            var stockList = Crms.SWmsApi.GetStockByCondition(new SWmsStockByCondition() { carrierId = data.AgvId.ToString() }, out var content);
            if (stockList == null)
            {
                res.message = $"获取AGV{data.AgvId}货物失败 {content}";
                return res;
            }

            var stock = stockList.FirstOrDefault();
            data.MaterialId = stock?.goodsCode;
            data.MaterialBarcode = stock?.attribute1;

            res.code = 20000;
            res.data = data;
            res.message = Langs.Successful;
            return res;
        }

        protected override WebMgrResponse PmsTaskInfoCheck(PmsTaskInfoDto pmsTaskInfo)
        {
            var webMgrResponse = new WebMgrResponse();

            //匹配取货点
            if (string.IsNullOrEmpty(pmsTaskInfo.LocationCode))
            {
                if (string.IsNullOrEmpty(pmsTaskInfo.WareHouseCode))
                {
                    webMgrResponse.message = Langs.SourceWarehouseAndSourceLocationAllEmpty;
                    return webMgrResponse;
                }

                string location = Crms.SWmsApi.GetLocationIdOut(pmsTaskInfo.WareHouseCode, pmsTaskInfo.MaterialId);
                if (string.IsNullOrEmpty(location))
                {
                    webMgrResponse.message = $"{Langs.SourceLocationMatchFailed} {Langs.TipWareHouseCode}{pmsTaskInfo.WareHouseCode} {Langs.TipMaterialId}{pmsTaskInfo.MaterialId}";
                    return webMgrResponse;
                }
                pmsTaskInfo.LocationCode = location;
            }

            //查询取货点信息
            var uLocationData = Crms.SWmsApi.GetLocationStockByLocationId(pmsTaskInfo.LocationCode);
            if (uLocationData == null)
            {
                webMgrResponse.message = $"{Langs.SourceLocationGetInfoFailed} {pmsTaskInfo.LocationCode}";
                return webMgrResponse;
            }
            if (!uLocationData.Location.enable)
            {
                webMgrResponse.message = $"{Langs.SourceLocationNotEnable} {uLocationData.Location.locationId}";
                return webMgrResponse;
            }
            if (uLocationData.Stock == null || uLocationData.Location.locationStatus == "0")
            {
                webMgrResponse.message = $"{Langs.SourceLocationMustBeFilled} {uLocationData.Location.locationId}";
                return webMgrResponse;
            }
            if (!string.IsNullOrEmpty(uLocationData.Location.lockMark))
            {
                webMgrResponse.message = $"{Langs.SourceLocationOccupiedByTask} {uLocationData.Location.locationId} {uLocationData.Location.lockMark}";
                return webMgrResponse;
            }
            if (!Crms.SWmsApi.CheckLocation(uLocationData, out var blockLocation1))
            {
                webMgrResponse.message = $"{Langs.SourceLocationObstacleDetectionFailed} {uLocationData.Location.locationId} => {blockLocation1?.Location.locationId}";
                return webMgrResponse;
            }
            if (!Crms.SWmsApi.CheckLocationColumnNoTask(uLocationData, out var taskLocation1))
            {
                webMgrResponse.message = $"{Langs.SourceLocationHaveColumnTask} {uLocationData.Location.locationId} （{taskLocation1?.Location.locationId}{Langs.TaskID}{taskLocation1?.Location.lockMark}）";
                return webMgrResponse;
            }


            //匹配放货点
            if (string.IsNullOrEmpty(pmsTaskInfo.TargetLocationCode))
            {
                if (string.IsNullOrEmpty(pmsTaskInfo.TargetWareHouseCode))
                {
                    webMgrResponse.message = Langs.TargetWarehouseAndTargetLocationAllEmpty;
                    return webMgrResponse;
                }


                string location = Crms.SWmsApi.GetLocationIdIn(pmsTaskInfo.TargetWareHouseCode);
                if (string.IsNullOrEmpty(location))
                {
                    webMgrResponse.message = $"{Langs.TargetLocationMatchFailed} {Langs.TipWareHouseCode}{pmsTaskInfo.TargetWareHouseCode}";
                    return webMgrResponse;
                }
                pmsTaskInfo.TargetLocationCode = location;
            }

            //查询放货点信息
            var dLocationData = Crms.SWmsApi.GetLocationStockByLocationId(pmsTaskInfo.TargetLocationCode);
            if (dLocationData == null)
            {
                webMgrResponse.message = $"{Langs.TargetLocationGetInfoFailed} {pmsTaskInfo.TargetLocationCode}";
                return webMgrResponse;
            }
            if (!dLocationData.Location.enable)
            {
                webMgrResponse.message = $"{Langs.TargetLocationNotEnable} {dLocationData.Location.locationId}";
                return webMgrResponse;
            }
            if (dLocationData.Stock != null || dLocationData.Location.locationStatus == "0")
            {
                webMgrResponse.message = $"{Langs.TargetLocationMustBeEmpty} {dLocationData.Location.locationId}";
                return webMgrResponse;
            }
            if (!string.IsNullOrEmpty(dLocationData.Location.lockMark))
            {
                webMgrResponse.message = $"{Langs.TargetLocationOccupiedByTask} {dLocationData.Location.locationId} {dLocationData.Location.lockMark}";
                return webMgrResponse;
            }
            if (!Crms.SWmsApi.CheckLocation(dLocationData, out var blockLocation2))
            {
                webMgrResponse.message = $"{Langs.TargetLocationObstacleDetectionFailed} {dLocationData.Location.locationId} => {blockLocation2?.Location.locationId}";
                return webMgrResponse;
            }
            if (!Crms.SWmsApi.CheckLocationColumnNoTask(dLocationData, out var taskLocation2))
            {
                webMgrResponse.message = $"{Langs.TargetLocationHaveColumnTask} {dLocationData.Location.locationId} （{taskLocation2?.Location.locationId}{Langs.TaskID}{taskLocation2?.Location.lockMark}）";
                return webMgrResponse;
            }



            pmsTaskInfo.WareHouseCode = uLocationData.Location.warehouseId;
            pmsTaskInfo.TargetWareHouseCode = dLocationData.Location.warehouseId;

            webMgrResponse.code = 20000;
            webMgrResponse.data = pmsTaskInfo;
            return webMgrResponse;
        }

        protected override bool CheckLocaitonStatus(MidTaskInfoEx data, string[] posArray = null)
        {
            if (posArray == null)
            {
                posArray = data.ParametersDic.Values.ToArray().Distinct().ToArray();
            }

            for (int i = 0; i < posArray.Length; i++)
            {
                string location = posArray[i];

                var locationData = Crms.SWmsApi.GetLocationStockByLocationId(location);
                if (locationData == null)
                {
                    continue;
                }

                if (!string.IsNullOrEmpty(locationData.Location.lockMark))
                {
                    CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {Langs.TaskOccupancyCheckFailed} {locationData.Location.locationId} {locationData.Location.locationId}");
                    return false;
                }
            }

            return true;
        }
        protected override bool LockLocationStatus(MidTaskInfoEx data, string[] posArray = null)
        {
            if (posArray == null)
            {
                posArray = data.ParametersDic.Values.ToArray().Distinct().ToArray();
            }

            for (int i = 0; i < posArray.Length; i++)
            {
                string location = posArray[i];

                var locationData = Crms.SWmsApi.GetLocationStockByLocationId(location);
                if (locationData == null)
                {
                    continue;
                }

                bool result = Crms.SWmsApi.UpdateLocationLockMark(location, null, data.TaskId, out var content);
                CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {location} {Langs.TaskOccupancy}{result.ToChineseString()} {(result ? null : content)}");

                if (!result) return result;
            }

            return true;
        }
        protected override void RecoverLocationStatus(MidTaskInfoEx data, string[] posArray = null)
        {
            if (posArray == null)
            {
                posArray = data.ParametersDic.Values.ToArray().Distinct().ToArray();
            }

            for (int i = 0; i < posArray.Length; i++)
            {
                string location = posArray[i];

                var locationData = Crms.SWmsApi.GetLocationStockByLocationId(location);
                if (locationData == null)
                {
                    continue;
                }

                bool result = Crms.SWmsApi.UpdateLocationLockMark(location, data.TaskId, null, out var content);
                CLog.Instance.TaskLog.WriteInfo($"{data.TaskId} {location} {Langs.TaskOccupationRelease}{result.ToChineseString()} {(result ? null : content)}");
            }
        }
        public override bool UnlimitLocation(string taskId, string location)
        {
            lock (locker)
            {
                var locationData = Crms.SWmsApi.GetLocationStockByLocationId(location);
                if (locationData == null)
                {
                    CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} {Langs.LocationGetInfoFailed}");
                    return false;
                }
                if (string.IsNullOrWhiteSpace(locationData.Location.lockMark))
                {
                    CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} {Langs.TaskOccupationNoNeedRelease}");
                    return false;
                }
                if (locationData.Location.lockMark != taskId)
                {
                    CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} {Langs.TaskOccupationMismatch} {locationData.Location.lockMark}");
                    return false;
                }

                locationData.Location.locationStatus = LocationStatus.None.ToValue<int>().ToString();
                var result2 = Crms.SWmsApi.UpdateLocation(locationData.Location, out var content2);
                if (!result2)
                {
                    CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} {Langs.LocationStatusUpdateFailed} {(result2 ? null : content2)}");
                }

                var result = Crms.SWmsApi.UpdateLocationLockMark(location, taskId, null, out var content);
                CLog.Instance.TaskLog.WriteInfo($"{taskId} {location} {Langs.TaskOccupationRelease}{result.ToChineseString()} {(result ? null : content)}");

                return result;
            }
        }
        public override string QueryLocationIn(string taskId, string wHouse)
        {
            lock (locker)
            {
                var list = Crms.SWmsApi.GetLocationStockByWarehouseId(wHouse);
                if (list == null) return null;

                var data = list.FirstOrDefault(d => d.Location.lockMark == taskId);
                if (data != null)
                {
                    return data.Location.locationId;
                }

                var nextLocationData = Crms.SWmsApi.GetLocationStockIn(wHouse);
                if (nextLocationData == null)
                {
                    return null;
                }

                var nextLocation = nextLocationData.Location.locationId;
                bool result = Crms.SWmsApi.UpdateLocationLockMark(nextLocation, null, taskId, out var content);
                if (!result)
                {
                    return null;
                }

                return nextLocation;
            }
        }

        public override bool StockMoveLocationToAgv(TaskData taskData, StepData stepData, out string message)
        {
            lock (locker)
            {
                var agvStock = Crms.SWmsApi.GetStockByAgvId(taskData.Carrier, out var content);
                if (agvStock != null)
                {
                    var result2 = Crms.SWmsApi.StockOutAutoByAgvId(taskData.Carrier, taskData.TaskID, out content);
                    if (!result2)
                    {
                        message = $"清除AGV货物数据失败 {content}";
                        return false;
                    }
                }

                var data = Crms.SWmsApi.GetLocationStockByLocationId(stepData.WareHouseID);
                if (data == null)
                {
                    message = $"获取库位信息失败";
                    return false;
                }

                var result = Crms.SWmsApi.UpdateLocationLockMark(stepData.WareHouseID, data.Location.lockMark, null, out content);
                if (!result)
                {
                    message = $"任务占用解除失败 {(result ? null : content)}";
                    return false;
                }

                result = Crms.SWmsApi.StockMoveAutoLocationToAgv(stepData.WareHouseID, taskData.Carrier, taskData.TaskID, out content);

                message = $"库位取货{result.ToChineseString()} {data?.Stock?.goodsCode} {data?.Stock?.goodsNumber} {(result ? null : content)}";
                return result;
            }
        }
        public override bool StockMoveAgvToLocation(TaskData taskData, StepData stepData, out string message)
        {
            lock (locker)
            {
                var agvStock = Crms.SWmsApi.GetStockByAgvId(taskData.Carrier, out var content);
                if (agvStock == null)
                {
                    message = $"获取AGV货物数据失败 {content}";
                    return false;
                }

                var data = Crms.SWmsApi.GetLocationStockByLocationId(stepData.WareHouseID);
                if (data == null)
                {
                    message = $"获取库位信息失败";
                    return false;
                }

                var result = Crms.SWmsApi.UpdateLocationLockMark(stepData.WareHouseID, data.Location.lockMark, null, out content);
                if (!result)
                {
                    message = $"任务占用解除失败 {(result ? null : content)}";
                    return false;
                }

                result = Crms.SWmsApi.StockMoveAutoAgvToLocation(taskData.Carrier, stepData.WareHouseID, taskData.TaskID, out content);
                data = Crms.SWmsApi.GetLocationStockByLocationId(stepData.WareHouseID);

                message = $"库位放货{result.ToChineseString()} {data?.Stock?.goodsCode} {data?.Stock?.goodsNumber} {(result ? null : content)}";
                return result;
            }
        }
    }
}

