﻿using ProjectManagementSystem.Common.Extenions;
using ProjectManagementSystem.Common.Logger;
using ProjectManagementSystem.Common.Models.Crms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectManagementSystem.Common.Service
{
    public class SWmsApi
    {
        private RestApi restApi = new RestApi();

        public bool Get<T>(string routes, object pars, object body, out T data)
        {
            data = default(T);
            try
            {
                var url = Common.Function.AppSetting.GetCrmsApiUrl(routes);
                var res = restApi.Get<WebMgrResponse<T>>(url, pars, body, out string content);

                data = res.data;
                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }
            return false;
        }
        public bool Post<T>(string routes, object pars, object body, out T data, out string content)
        {
            data = default(T);
            content = null;
            try
            {
                var url = Common.Function.AppSetting.GetCrmsApiUrl(routes);
                var res = restApi.Post<WebMgrResponse<T>>(url, pars, body, out content);
                //CLog.Instance.SystemLog.WriteDebug($"Post {url}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");

                data = res.data;
                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }

            return false;
        }
        public bool Put<T>(string routes, object pars, object body, out T data, out string content)
        {
            data = default(T);
            content = null;
            try
            {
                var url = Common.Function.AppSetting.GetCrmsApiUrl(routes);
                var res = restApi.Put<WebMgrResponse<T>>(url, pars, body, out content);
                //CLog.Instance.SystemLog.WriteDebug($"Put {url}{Environment.NewLine}  {JsonConvert.SerializeObject(body)}{Environment.NewLine}  {content}");

                data = res.data;
                return res.code == 20000;
            }
            catch (Exception ex)
            {
                CLog.Instance.SystemLog.WriteError(ex.Message);
            }

            return false;
        }


        public List<SWmsWarehouse> GetAllWarehouse()
        {
            if (!Get("wms/warehouse-config/getAllWarehouse", null, null, out List<SWmsWarehouse> dataList)) return null;
            return dataList;
        }
        public List<SWmsArea> GetAllArea()
        {
            if (!Get("wms/area-config/getAllArea", null, null, out List<SWmsArea> dataList)) return null;
            return dataList;
        }
        public List<SWmsLocationType> GetAllLocationType()
        {
            if (!Get("wms/location-type-config/getAllLocationType", null, null, out List<SWmsLocationType> dataList)) return null;
            return dataList;
        }
        public List<SWmsLocationStatus> GetAllLocationStatus()
        {
            if (!Get("wms/location-status-config/getAllLocationStatus", null, null, out List<SWmsLocationStatus> dataList)) return null;
            return dataList;
        }
        public List<SWmsFreightShape> GetAllFreightShape()
        {
            if (!Get("wms/freight-shape-config/getAllFreightShape", null, null, out List<SWmsFreightShape> dataList)) return null;
            return dataList;
        }
        public List<SWmsGoodsType> GetAllGoodsType()
        {
            if (!Get("wms/goods-type-config/getAllGoodsType", null, null, out List<SWmsGoodsType> dataList)) return null;
            return dataList;
        }
        public List<SWmsAttribute> GetAllAttribute()
        {
            if (!Get("wms/attribute-config/getAllAttribute", null, null, out List<SWmsAttribute> dataList)) return null;
            return dataList;
        }


        public bool AddWarehouse(SWmsWarehouse data, out string content)
        {
            return Post("warehouse-config/addWarehouse", null, data, out object _, out content);
        }
        public bool UpdateWarehouse(SWmsWarehouse data, out string content)
        {
            return Post("warehouse-config/updateWarehouse", null, data, out object _, out content);
        }
        public bool AddArea(SWmsArea data, out string content)
        {
            return Post("area-config/addArea", null, data, out object _, out content);
        }
        public bool UpdateArea(SWmsArea data, out string content)
        {
            return Post("area-config/updateArea", null, data, out object _, out content);
        }
        public bool AddLocationType(SWmsLocationType data, out string content)
        {
            return Post("location-type/addLocationType", null, data, out object _, out content);
        }
        public bool UpdateLocationType(SWmsLocationType data, out string content)
        {
            return Post("location-type/updateLocationType", null, data, out object _, out content);
        }
        public bool AddLocationStatus(SWmsLocationStatus data, out string content)
        {
            return Post("location-status/addLocationStatus", null, data, out object _, out content);
        }
        public bool UpdateLocationStatus(SWmsLocationStatus data, out string content)
        {
            return Post("location-status/updateLocationStatus", null, data, out object _, out content);
        }
        public bool AddFreightShape(SWmsFreightShape data, out string content)
        {
            return Post("freight-shape/", null, data, out object _, out content);
        }
        public bool UpdateFreightShape(SWmsFreightShape data, out string content)
        {
            return Put("freight-shape/", null, data, out object _, out content);
        }
        public bool AddGoodsType(SWmsGoodsType data, out string content)
        {
            return Post("goods-type/addGoodsType", null, data, out object _, out content);
        }
        public bool UpdateGoodsType(SWmsGoodsType data, out string content)
        {
            return Post("goods-type/updateGoodsType", null, data, out object _, out content);
        }
        public bool AddAttribute(SWmsAttribute data, out string content)
        {
            return Post("attribute-config/addAttribute", null, data, out object _, out content);
        }
        public bool UpdateAttribute(SWmsAttribute data, out string content)
        {
            return Post("attribute-config/updateAttribute", null, data, out object _, out content);
        }


        public bool AddRack(SWmsRack data, out string content)
        {
            return Post("wms/rack/addRack", null, data, out object _, out content);
        }
        public bool UpdateRack(SWmsRack data, out string content)
        {
            return Post("wms/rack/updateRack", null, data, out object _, out content);
        }
        public bool DeleteRack(string rackCode, out string content)
        {
            return Post("wms/rack/deleteRack", null, new { rackCode }, out object _, out content);
        }
        public List<SWmsRack> GetAllRack()
        {
            if (!Get("wms/rack/getAllRack", null, null, out List<SWmsRack> dataList)) return null;
            return dataList;
        }
        public List<SWmsRack> GetEmptyRack()
        {
            if (!Get("wms/rack/getEmptyRack", null, null, out List<SWmsRack> dataList)) return null;
            return dataList;
        }


        public bool AddGoods(SWmsGoods data, out string content)
        {
            return Post("wms/goods/addGoods", null, data, out object _, out content);
        }
        public bool UpdateGoods(SWmsGoods data, out string content)
        {
            return Post("wms/goods/updateGoods", null, data, out object _, out content);
        }
        public bool DeleteGoods(string goodsCode, out string content)
        {
            return Post("wms/goods/deleteGoods", null, new { goodsCode }, out object _, out content);
        }
        public List<SWmsGoods> GetAllGoods()
        {
            if (!Get("wms/goods/getAllGoods", null, null, out List<SWmsGoods> dataList)) return null;
            return dataList;
        }
        public List<SWmsGoods> GetGoodsByCondition(SWmsGoodsByCondition data, out string content)
        {
            if (!Post("wms/goods/getGoodsByCondition", null, data, out List<SWmsGoods> dataList, out content)) return null;
            return dataList;
        }


        public bool AddLocation(SWmsLocation data, out string content)
        {
            return Post("wms/location/addLocation", null, data, out object _, out content);
        }
        public bool UpdateLocation(SWmsLocation data, out string content)
        {
            return Post("wms/location/updateLocation", null, data, out object _, out content);
        }
        public bool DeleteLocation(string locationId, out string content)
        {
            return Post("wms/location/deleteLocation", null, new { locationId }, out object _, out content);
        }
        public List<SWmsLocation> GetAllLocation()
        {
            if (!Get("wms/location/getAllLocation", null, null, out List<SWmsLocation> dataList)) return null;
            return dataList;
        }
        public List<SWmsLocation> GetLocationByCondition(SWmsLocationByCondition data, out string content)
        {
            if (!Post("wms/location/getLocationByCondition", null, data, out List<SWmsLocation> dataList, out content)) return null;
            return dataList;
        }
        public List<SWmsWarehouseGroup> GetAllWarehouseGroup()
        {
            if (!Get("wms/location/getAllWarehouseGroup", null, null, out List<SWmsWarehouseGroup> dataList)) return null;
            return dataList;
        }
        public bool UpdateLocationLockMark(SWmsLocationLockMark data, out string content)
        {
            return Post("wms/location/updateLocationLockMark", null, data, out object _, out content);
        }
        public List<SWmsLocation> GetEmptyLocation()
        {
            if (!Get("wms/location/getEmptyLocation", null, null, out List<SWmsLocation> dataList)) return null;
            return dataList;
        }


        public bool StockIn(SWmsStock data, out string content)
        {
            return Post("wms/stock/stockIn", null, data, out object _, out content);
        }
        public bool StockOut(SWmsStockOut data, out string content)
        {
            return Post("wms/stock/stockOut", null, data, out object _, out content);
        }
        public bool StockMove(SWmsStockMove data, out string content)
        {
            return Post("wms/stock/stockMove", null, data, out object _, out content);
        }
        public bool UpdateStock(SWmsStock data, out string content)
        {
            return Post("wms/stock/updateStock", null, data, out object _, out content);
        }
        public bool StockOutById(SWmsStockOutById data, out string content)
        {
            return Post("wms/stock/stockOutById", null, data, out object _, out content);
        }
        public List<SWmsStock> GetStockByCondition(SWmsStockByCondition data, out string content)
        {
            if (!Post("wms/stock/getStockByCondition", null, data, out List<SWmsStock> dataList, out content)) return null;
            return dataList;
        }
        public SWmsStockRecordPage StockRecordFindPage(SWmsStockRecordCondition data, out string content)
        {
            if (!Post("wms/stock-record/findPage", null, data, out SWmsStockRecordPage dataList, out content)) return null;
            return dataList;
        }



        //function
        public bool UpdateLocationLockMark(string locationId, string beforeLockMark, string afterLockMark, out string content)
        {
            return UpdateLocationLockMark(new SWmsLocationLockMark() { locationId = locationId, beforeLockMark = beforeLockMark, afterLockMark = afterLockMark }, out content);
        }
        public bool StockSave(SWmsStock stock, out string message)
        {
            string content = null;
            string supporterIdDesc = null;
            var supporterType = stock.supporterType;
            var supporterId = stock.supporterId;
            List<SWmsStock> stockList = null;

            switch (supporterType)
            {
                case SWmsESupporterType.Location:
                    stockList = GetStockByCondition(new SWmsStockByCondition() { locationId = supporterId }, out content);
                    supporterIdDesc = supporterId;
                    break;
                case SWmsESupporterType.Carrier:
                    stockList = GetStockByCondition(new SWmsStockByCondition() { carrierId = supporterId }, out content);
                    supporterIdDesc = $"{supporterType}:{supporterId}";
                    break;
                case SWmsESupporterType.Elevator:
                    stockList = GetStockByCondition(new SWmsStockByCondition() { elevatorId = supporterId }, out content);
                    supporterIdDesc = $"{supporterType}:{supporterId}";
                    break;
                default:
                    break;
            }

            if (stockList == null)
            {
                message = $"获取库存失败 {content}"; ;
                return false;
            }

            var result = false;
            var oStock = stockList.FirstOrDefault();

            //处理库存
            if (string.IsNullOrWhiteSpace(stock.goodsCode))
            {
                //没有库存直接返回
                if (oStock == null)
                {
                    message = $"{supporterIdDesc} 货物已清空";
                    return true;
                }

                //清空货物
                result = StockOut(new SWmsStockOut() { supporterType = supporterType, supporterId = supporterId, recordType = SWmsERecordType.ManualOut, operationSource = stock.operationSource }, out content);
                message = $"{supporterIdDesc} 清空货物 {result.ToChineseString()} {(result ? null : content)}";

                return result;
            }
            else
            {
                //添加货物
                var goodsList = GetGoodsByCondition(new SWmsGoodsByCondition() { goodsCode = stock.goodsCode }, out content);
                if (goodsList == null)
                {
                    message = $"获取货物失败 {stock.goodsCode} {content}";
                    return false;
                }

                if (!goodsList.Exists(d => d.goodsCode == stock.goodsCode))
                {
                    var freightShapeList = GetAllFreightShape();
                    if (freightShapeList == null)
                    {
                        message = "获取所有模型失败";
                        return false;
                    }

                    if (freightShapeList.Count == 0)
                    {
                        message = "找不到任何货物模型";
                        return false;
                    }

                    var data2 = new SWmsGoods();
                    data2.goodsCode = stock.goodsCode;
                    data2.goodsName = stock.goodsCode;
                    data2.goodsShape = freightShapeList.FirstOrDefault()?.id ?? 0;

                    result = AddGoods(data2, out content);
                    if (!result)
                    {
                        message = $"添加货物失败 {content}";
                        return false;
                    }
                }
            }

            //设置库存
            if (oStock == null)
            {
                result = StockIn(stock, out content);
                message = $"{supporterIdDesc} 设置货物 {stock.goodsCode} {stock.goodsNumber} {result.ToChineseString()} {(result ? null : content)}";

                return result;
            }
            else
            {
                //清除多余库存
                var unnecessaryStocks = stockList.Where(d => d.id != oStock.id);
                foreach (var item in unnecessaryStocks)
                {
                    result = StockOutById(new SWmsStockOutById() { id = item.id, recordType = SWmsERecordType.ManualOut, operationSource = stock.operationSource }, out content);
                    if (!result)
                    {
                        message = $"{supporterIdDesc} 清除多余货物 {item.id} {result.ToChineseString()} {(result ? null : content)}";
                        return false;
                    }
                }

                //库存一致时直接返回
                if (oStock.goodsCode == stock.goodsCode
                    && oStock.goodsNumber == stock.goodsNumber
                    && oStock.rackCode == stock.rackCode
                    && oStock.taskId == stock.taskId
                    && oStock.attribute1 == stock.attribute1
                    && oStock.attribute2 == stock.attribute2
                    && oStock.attribute3 == stock.attribute3
                    && oStock.attribute4 == stock.attribute4
                    && oStock.attribute5 == stock.attribute5
                    && oStock.attribute6 == stock.attribute6
                    && oStock.attribute7 == stock.attribute7
                    && oStock.attribute8 == stock.attribute8
                    && oStock.attribute9 == stock.attribute9
                    && oStock.attribute10 == stock.attribute10
                    && oStock.attribute11 == stock.attribute11
                    && oStock.attribute12 == stock.attribute12
                    && oStock.attribute13 == stock.attribute13
                    && oStock.attribute14 == stock.attribute14
                    && oStock.attribute15 == stock.attribute15
                    && oStock.attribute16 == stock.attribute16
                    && oStock.attribute17 == stock.attribute17
                    && oStock.attribute18 == stock.attribute18
                    && oStock.attribute19 == stock.attribute19
                    && oStock.attribute20 == stock.attribute20)
                {
                    message = $"{supporterIdDesc} 货物已设置 {stock.goodsCode} {stock.goodsNumber}";
                    return true;
                }

                //更新库存
                stock.id = oStock.id;
                stock.recordType = SWmsERecordType.Update;
                result = UpdateStock(stock, out content);

                message = $"{supporterIdDesc} 更新货物 {stock.goodsCode} {stock.goodsNumber} {result.ToChineseString()} {(result ? null : content)}";
                return result;
            }
        }
        public bool StockOutAutoByLocationId(string locationId, string taskId, out string content)
        {
            return StockOut(new SWmsStockOut() { supporterType = SWmsESupporterType.Location, supporterId = locationId, recordType = SWmsERecordType.AutoOut, operationSource = "pms", taskId = taskId }, out content);
        }
        public bool StockOutAutoByAgvId(int agvId, string taskId, out string content)
        {
            return StockOut(new SWmsStockOut() { supporterType = SWmsESupporterType.Carrier, supporterId = agvId.ToString(), recordType = SWmsERecordType.AutoOut, operationSource = "pms", taskId = taskId }, out content);
        }
        public bool StockMoveAutoLocationToAgv(string locationId, int agvId, string taskId, out string content)
        {
            var data = new SWmsStockMove()
            {
                currentSupporterType = SWmsESupporterType.Location,
                currentSupporterId = locationId,
                targetSupporterType = SWmsESupporterType.Carrier,
                targetSupporterId = agvId.ToString(),
                recordType = SWmsERecordType.AutoMove,
                operationSource = "pms",
                taskId = taskId
            };

            return StockMove(data, out content);
        }
        public bool StockMoveAutoAgvToLocation(int agvId, string locationId, string taskId, out string content)
        {
            var data = new SWmsStockMove()
            {
                currentSupporterType = SWmsESupporterType.Carrier,
                currentSupporterId = agvId.ToString(),
                targetSupporterType = SWmsESupporterType.Location,
                targetSupporterId = locationId,
                recordType = SWmsERecordType.AutoMove,
                operationSource = "pms",
                taskId = taskId
            };

            return StockMove(data, out content);
        }

        public SWmsStock GetStockByAgvId(int agvId, out string content)
        {
            return GetStockByCondition(new SWmsStockByCondition() { carrierId = agvId.ToString() }, out content)?.FirstOrDefault();
        }
        public List<SWmsLocationStock> GetLocationStock(SWmsLocationByCondition locationByCondition, SWmsStockByCondition stockByCondition)
        {
            var locationList = GetLocationByCondition(locationByCondition, out var content);
            if (locationList == null || locationList.Count == 0) return null;

            var goodsList = GetStockByCondition(stockByCondition, out content);
            if (goodsList == null) return null;

            return locationList.GroupJoin(goodsList, o => o.locationId, i => i.supporterId, (o, i) => new SWmsLocationStock() { Location = o, Stock = i.FirstOrDefault(), StockList = i }).ToList();
        }
        public List<SWmsLocationStock> GetLocationStock(string warehouseId, string areaId, string locationId)
        {
            var locationByCondition = new SWmsLocationByCondition() { warehouseId = warehouseId, areaId = areaId, locationId = locationId };
            var stockByCondition = new SWmsStockByCondition() { warehouseId = warehouseId, areaId = areaId, locationId = locationId };
            return GetLocationStock(locationByCondition, stockByCondition);
        }
        public List<SWmsLocationStock> GetLocationStockByWarehouseId(string warehouseId)
        {
            var locationByCondition = new SWmsLocationByCondition() { warehouseId = warehouseId };
            var stockByCondition = new SWmsStockByCondition() { warehouseId = warehouseId };
            return GetLocationStock(locationByCondition, stockByCondition);
        }
        public SWmsLocationStock GetLocationStockByLocationId(string locationId)
        {
            var locationByCondition = new SWmsLocationByCondition() { locationId = locationId };
            var stockByCondition = new SWmsStockByCondition() { locationId = locationId };
            return GetLocationStock(locationByCondition, stockByCondition)?.FirstOrDefault();
        }
        public SWmsLocationStock GetLocationStockOut(string warehouseId, string material = null)
        {
            var wLocationDatas = GetLocationStockByWarehouseId(warehouseId);
            if (wLocationDatas == null) return null;

            var locationData = wLocationDatas
                .Where(d => d.Location.enable && string.IsNullOrWhiteSpace(d.Location.lockMark) && d.Stock != null && d.Location.locationStatus != "0")
                .Where(d => material == null ? true : d.Stock.goodsCode == material)
                .OrderBy(d => d.Location.column)
                .ThenByDescending(d => d.Location.row)
                .ThenBy(d => d.Stock.updateTime)
                .FirstOrDefault(d => CheckLocation(wLocationDatas, d, out var _) && CheckLocationColumnNoTask(wLocationDatas, d, out var _));

            return locationData;
        }
        public SWmsLocationStock GetLocationStockIn(string warehouseId)
        {
            var wLocationDatas = GetLocationStockByWarehouseId(warehouseId);
            if (wLocationDatas == null) return null;

            var locationData = wLocationDatas
                .Where(d => d.Location.enable && string.IsNullOrWhiteSpace(d.Location.lockMark) && d.Stock == null && d.Location.locationStatus != "0")
                .OrderBy(d => d.Location.column)
                .ThenByDescending(d => d.Location.row)
                .FirstOrDefault(d => CheckLocation(wLocationDatas, d, out var _) && CheckLocationColumnNoTask(wLocationDatas, d, out var _));

            return locationData;
        }
        public SWmsLocationStock GetLocationStockIn(string warehouseId, string material)
        {
            var wLocationDatas = GetLocationStockByWarehouseId(warehouseId);
            if (wLocationDatas == null) return null;

            var materialColumns = wLocationDatas
                .GroupBy(d => d.Location.column)
                .Where(d =>
                {
                    var tempOrdered = d.OrderBy(s => s.Location.row);
                    if (tempOrdered?.FirstOrDefault()?.Stock != null)
                    {
                        //该列全满
                        return false;
                    }

                    var tempLocationStock = tempOrdered?.FirstOrDefault(s => s.Stock != null);
                    if (tempLocationStock == null)
                    {
                        //该列全空
                        return true;
                    }

                    //该列库位物料一致
                    return tempLocationStock?.Stock?.goodsCode == material;
                })
                .OrderBy(d => d.OrderByDescending(s => s.Location.row)?.FirstOrDefault(s => s.Stock == null)?.Location.row)
                .Select(d => d.Key)
                .Distinct()
                .ToList();

            var locationData = wLocationDatas
                .Where(d => materialColumns.Contains(d.Location.column) && d.Location.enable && string.IsNullOrWhiteSpace(d.Location.lockMark) && d.Stock == null && d.Location.locationStatus != "0")
                .OrderBy(d => materialColumns.IndexOf(d.Location.column))
                .ThenByDescending(d => d.Location.row)
                .FirstOrDefault(d => CheckLocation(wLocationDatas, d, out var _) && CheckLocationColumnNoTask(wLocationDatas, d, out var _));

            return locationData;
        }
        public string GetLocationIdOut(string warehouseId, string material = null)
        {
            return GetLocationStockOut(warehouseId, material)?.Location?.locationId;
        }
        public string GetLocationIdIn(string warehouseId)
        {
            return GetLocationStockIn(warehouseId)?.Location?.locationId;
        }
        public bool CheckLocation(SWmsLocationStock data, out SWmsLocationStock blockLocation)
        {
            blockLocation = null;
            var wLocationDatas = GetLocationStockByWarehouseId(data.Location.warehouseId);
            if (wLocationDatas == null) return false;

            return CheckLocation(wLocationDatas, data, out blockLocation);
        }
        public bool CheckLocationColumnNoTask(SWmsLocationStock data, out SWmsLocationStock taskLocation)
        {
            taskLocation = null;
            var wLocationDatas = GetLocationStockByWarehouseId(data.Location.warehouseId);
            if (wLocationDatas == null) return false;

            return CheckLocationColumnNoTask(wLocationDatas, data, out taskLocation);
        }
        public bool CheckLocation(IEnumerable<SWmsLocationStock> wLocationDatas, SWmsLocationStock data, out SWmsLocationStock blockLocation)
        {
            blockLocation = null;
            if (data == null) return false;
            if (string.IsNullOrWhiteSpace(data.Location.column) && string.IsNullOrWhiteSpace(data.Location.row)) return true;

            //检查出库的线路上是否都是空状态（障碍检测）
            blockLocation = wLocationDatas
                .FirstOrDefault(d => d.Location.column == data.Location.column
                && d.Location.row.CompareTo(data.Location.row) < 0
                && (d.Stock != null || !d.Location.enable || d.Location.locationStatus == "0"));

            return blockLocation == null;
        }
        public bool CheckLocationColumnNoTask(IEnumerable<SWmsLocationStock> wLocationDatas, SWmsLocationStock data, out SWmsLocationStock taskLocation)
        {
            taskLocation = null;
            if (data == null) return false;
            if (string.IsNullOrWhiteSpace(data.Location.column) && string.IsNullOrWhiteSpace(data.Location.row)) return true;

            //检查库位相同列上是否全部没有任务（同列任务检测）
            taskLocation = wLocationDatas
                .FirstOrDefault(d => d.Location.column == data.Location.column
                && d.Location.row != data.Location.row
                && !string.IsNullOrWhiteSpace(d.Location.lockMark));

            return taskLocation == null;
        }
    }
}
